'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.idFetcher = idFetcher;
exports.typeResolver = typeResolver;
exports.nodeInterfaceMapper = nodeInterfaceMapper;

var _graphqlRelay = require('graphql-relay');

var _nodeMapper = require('./nodeMapper');

var _nodeMapper2 = _interopRequireDefault(_nodeMapper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Id Global Fetcher
 *
 * @param thinky
 * @param nodeTypeMapper
 * @returns {Function}
 */
function idFetcher(Models, nodeTypeMapper) {
  var _this = this;

  return function () {
    var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(globalId, context) {
      var _fromGlobalId, type, id, nodeType, res, model, result;

      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _fromGlobalId = (0, _graphqlRelay.fromGlobalId)(globalId), type = _fromGlobalId.type, id = _fromGlobalId.id;
              nodeType = nodeTypeMapper.item(type);

              if (!(nodeType && typeof nodeType.resolve === 'function')) {
                _context.next = 8;
                break;
              }

              _context.next = 5;
              return Promise.resolve(nodeType.resolve(globalId, context));

            case 5:
              res = _context.sent;

              res.__graphqlType__ = type;
              return _context.abrupt('return', res);

            case 8:
              model = Object.keys(Models).find(function (model) {
                return model === type;
              });
              result = null;

              if (!model) {
                _context.next = 16;
                break;
              }

              _context.next = 13;
              return Models[model].get(id).run();

            case 13:
              result = _context.sent;
              _context.next = 17;
              break;

            case 16:
              result = nodeType;

            case 17:
              if (!result) {
                _context.next = 19;
                break;
              }

              return _context.abrupt('return', nodeType.type);

            case 19:
              return _context.abrupt('return', result);

            case 20:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();
}

/**
 * Type Resolver
 *
 * @param nodeTypeMapper
 * @returns {Function}
 */
function typeResolver(nodeTypeMapper) {
  return function (obj) {
    var type = obj.__graphqlType__ || typeof obj.getModel === 'function' ? obj.getModel().getModelName() : obj.name;

    if (!type) {
      throw new Error('Unable to determine type of ' + (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) + '. ' + 'Either specify a resolve function in \'NodeTypeMapper\' object, or specify \'__graphqlType__\' property on object.');
    }

    var nodeType = nodeTypeMapper.item(type);
    return nodeType && nodeType.type || null;
  };
}

/**
 * Export node definitions
 *
 * @param Database
 * @returns {{nodeTypeMapper: NodeTypeMapper}}
 */
function nodeInterfaceMapper(Models) {
  var nodeTypeMapper = new _nodeMapper2.default();
  var nodeObjects = (0, _graphqlRelay.nodeDefinitions)(idFetcher(Models, nodeTypeMapper), typeResolver(nodeTypeMapper));

  return _extends({
    nodeTypeMapper: nodeTypeMapper
  }, nodeObjects);
}